package com.sun.org.apache.xml.internal.security.utils;

import java.io.IOException;
import java.io.OutputStream;
import java.io.FilterOutputStream;

public class UnsyncBufferedOutputStream extends FilterOutputStream
{
    protected byte[] buffer;
    protected int count;
    
    public UnsyncBufferedOutputStream(OutputStream out) {
        super(out);
        this.buffer = new byte[8192];
    }
    
    public UnsyncBufferedOutputStream(OutputStream out, int n) {
        super(out);
        if (n <= 0) {
            throw new IllegalArgumentException("size must be > 0");
        }
        this.buffer = new byte[n];
        return;
    }
    
    @Override
    public void flush() throws IOException {
        this.flushInternal();
        this.out.flush();
    }
    
    @Override
    public void write(byte[] b, int off, int len) throws IOException {
        if (len >= this.buffer.length) {
            this.flushInternal();
            this.out.write(b, off, len);
            return;
        }
        if (len >= this.buffer.length - this.count) {
            this.flushInternal();
        }
        System.arraycopy(b, off, this.buffer, this.count, len);
        this.count += len;
        return;
    }
    
    @Override
    public void write(int n) throws IOException {
        if (this.count == this.buffer.length) {
            this.out.write(this.buffer, 0, this.count);
            this.count = 0;
        }
        this.buffer[this.count++] = (byte)n;
    }
    
    private void flushInternal() throws IOException {
        if (this.count > 0) {
            this.out.write(this.buffer, 0, this.count);
            this.count = 0;
        }
    }
}
